@@ -38,21 +38,20 @@
 
 #include "r300_fragprog_common.h"
 
-#include "shader/prog_parameter.h"
-#include "shader/prog_print.h"
+#include "program/prog_print.h"
 
 #include "compiler/radeon_compiler.h"
 
 #include "radeon_mesa_to_rc.h"
 
 
-static GLuint build_dtm(GLuint depthmode)
+static GLuint build_dts(GLuint depthmode)
 {
 	switch(depthmode) {
 	default:
-	case GL_LUMINANCE: return 0;
-	case GL_INTENSITY: return 1;
-	case GL_ALPHA: return 2;
+	case GL_LUMINANCE: return RC_SWIZZLE_XYZZ;
+	case GL_INTENSITY: return RC_SWIZZLE_XYZW;
+	case GL_ALPHA: return RC_SWIZZLE_WWWW;
 	}
 }
 
@@ -78,7 +77,7 @@ static void build_state(
 		if (fp->Base.ShadowSamplers & (1 << unit)) {
 			struct gl_texture_object* tex = r300->radeon.glCtx->Texture.Unit[unit]._Current;
 
-			state->unit[unit].depth_texture_mode = build_dtm(tex->DepthMode);
+			state->unit[unit].depth_texture_swizzle = build_dts(tex->DepthMode);
 			state->unit[unit].texture_compare_func = build_func(tex->CompareFunc);
 		}
 	}
@@ -209,17 +208,24 @@ static void allocate_hw_inputs(
 }
 
 
-static void translate_fragment_program(GLcontext *ctx, struct r300_fragment_program_cont *cont, struct r300_fragment_program *fp)
+static void translate_fragment_program(struct gl_context *ctx, struct r300_fragment_program_cont *cont, struct r300_fragment_program *fp)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
 	struct r300_fragment_program_compiler compiler;
 
+        memset(&compiler, 0, sizeof(compiler));
 	rc_init(&compiler.Base);
 	compiler.Base.Debug = (RADEON_DEBUG & RADEON_PIXEL) ? GL_TRUE : GL_FALSE;
 
 	compiler.code = &fp->code;
 	compiler.state = fp->state;
-	compiler.is_r500 = (r300->radeon.radeonScreen->chip_family >= CHIP_FAMILY_RV515) ? GL_TRUE : GL_FALSE;
+	compiler.enable_shadow_ambient = GL_TRUE;
+	compiler.Base.is_r500 = (r300->radeon.radeonScreen->chip_family >= CHIP_FAMILY_RV515) ? GL_TRUE : GL_FALSE;
+	compiler.Base.disable_optimizations = 0;
+	compiler.Base.has_half_swizzles = 1;
+	compiler.Base.max_temp_regs = (compiler.Base.is_r500) ? 128 : 32;
+	compiler.Base.max_constants = compiler.Base.is_r500 ? 256 : 32;
+	compiler.Base.max_alu_insts = compiler.Base.is_r500 ? 512 : 64;
 	compiler.OutputDepth = FRAG_RESULT_DEPTH;
 	memset(compiler.OutputColor, 0, 4 * sizeof(unsigned));
 	compiler.OutputColor[0] = FRAG_RESULT_COLOR;
@@ -240,7 +246,7 @@ static void translate_fragment_program(GLcontext *ctx, struct r300_fragment_prog
 
 	r3xx_compile_fragment_program(&compiler);
 
-	if (compiler.is_r500) {
+	if (compiler.Base.is_r500) {
 		/* We need to support the non-KMS DRM interface, which
 		 * artificially limits the number of instructions and
 		 * constants which are available to us.
@@ -272,7 +278,7 @@ static void translate_fragment_program(GLcontext *ctx, struct r300_fragment_prog
 	rc_destroy(&compiler.Base);
 }
 
-struct r300_fragment_program *r300SelectAndTranslateFragmentShader(GLcontext *ctx)
+struct r300_fragment_program *r300SelectAndTranslateFragmentShader(struct gl_context *ctx)
 {
 	r300ContextPtr r300 = R300_CONTEXT(ctx);
 	struct r300_fragment_program_cont *fp_list;
