@@ -450,7 +450,8 @@ struct r100_context {
 
 #define RADEON_OLD_PACKETS 1
 
-extern GLboolean r100CreateContext( const __GLcontextModes *glVisual,
+extern GLboolean r100CreateContext( gl_api api,
+				    const struct gl_config *glVisual,
 				    __DRIcontext *driContextPriv,
 				    void *sharedContextPrivate);
 
